/*
ͻ
 What's it called: EPMTeX v2.0  (tex.e)                                     
                                                                            
 What does it do:  The E source code for the EPMTeX enhancements for EPM.   
                                                                            
 Who and When:     Jon Hacker   1/93 - 8/94                                 
                   Joachim Koenen 8/94       (syntax assist)                
                   Thorsten Domsalla 2/93    (menus, batch files)           
                   Rodney Korte 7/93 -6/94                                  
                                                                            
ͼ
*/
/*
  Comments/Questions (by RK):
  - Certainly some of the code could be cleaned up (optimized).  Things get
    pretty bogged down when there are lots of files in the ring.
  - Using file id's would probably be a much more elegant way of handling
    files in the ring than the 'edit/n' that is used now.
  - It might be handy to go back to previous errors, not just forward.
    I am definitely *not* up to that task.
  - Why won't the circles stay around? (See procedure tex_find_errors.)
  - Perhaps a "Load all" menu option that scans the current file for
    \input commands and loads those files into the editor would be handy?
  - Is there a need for the Automatic LOG file delete?
*/

/*
Ŀ
  Include the standard stuff we need since we are compiling separately      
  from epm.ex and extra.ex.                                                 

*/
compile if not defined(SMALL)  -- Being compiled separately
  include 'stdconst.e'
  include 'colors.e'
  define INCLUDING_FILE = 'TEX.E'
  tryinclude 'MYCNF.E'
  compile if not defined(SITE_CONFIG)
    const SITE_CONFIG = 'SITECNF.E'
  compile endif
  compile if SITE_CONFIG
    tryinclude SITE_CONFIG
  compile endif
  compile if not defined(NLS_LANGUAGE)
    const NLS_LANGUAGE = 'ENGLISH'
  compile endif
  include NLS_LANGUAGE'.e'
  compile if not defined(EPM_POINTER)
    const EPM_POINTER = TEXT_POINTER
  compile endif
compile endif

/*
Ŀ
  Include the user edited environment file (texenv.e) that describes the    
      location of required TeX programs and files.                          

*/

include 'texenv.e'


/*
Ŀ
  EPMTeX menu messages - alter these for alternate language support         

*/

const
TEX_BAR__MSG =          \1'Menus related to TeX support'
TEX_TEX__MSG =          \1'TeX the current file (or Master file, if applicable)'
TEX_LOCATE_ERROR__MSG = \1'Locate next error from LOG file'
TEX_VIEW_LOG__MSG =     \1'View the LOG file associated with the current file'
TEX_VIEW_DVI__MSG =     \1'Preview the DVI file associated with the current file'
TEX_PRINT__MSG =        \1'Print the DVI file associated with the current file'
TEX_DEL_LOG__MSG =      \1'Delete the LOG file associated with the current file'
TEX_DEL_DVI__MSG =      \1'Delete the DVI file associated with the current file'
TEX_FORMAT__MSG =       \1'Format to be used when TeXing'
TEX_ORIENT__MSG =       \1'Orientation with which to Preview and Print'
TEX_ORIENT_AUTO__MSG=   \1'Auto select of orientation based on directive in 1st line of file'
TEX_ORIENT_PORT__MSG=   \1'Force Portrait orientation for Preview and Print'
TEX_ORIENT_LAND__MSG=   \1'Force Landscape orientation for Preview and Print'
TEX_AUTO_DEL_LOG__MSG = \1'Automatic delete of the associated LOG file after View or Print'
TEX_AUTO_SAVE__MSG =    \1'Automatic smart save of TeX file before TeXing document.'
TEX_SYNTAX__MSG =       \1'Global syntax expansion mode (expand_on)'
TEX_DEBUG__MSG =        \1'Verbose mode for troubleshooting TeX commands'
TEX_FORMAT_AUTO__MSG=   \1'Auto select of format based on directive in 1st line of file'
TEX_ABOUT_BOX__MSG=     \1'About EPMTeX 2.0'

/*
Ŀ
  Start of EPMTeX fixed code                                                

*/
-- Set default conditions if not set in mycnf.e
-- TEX_SYNTAX_ASSIST and TEX_QUICK_KEYS also in texkeys.e

compile if not defined(TEX_SYNTAX_ASSIST)
const  TEX_SYNTAX_ASSIST = 1               -- default is to include
compile endif                              -- TeX syntax expansion

compile if not defined(TEX_QUICK_KEYS)      -- S_F11 = TeX document
const   TEX_QUICK_KEYS = 1                  -- S_F12 = Find Next Error
compile endif                               -- Default is to have them

compile if not defined(TEX_MENU_ALWAYS_ON)
const   TEX_MENU_ALWAYS_ON = 0              --  default is to grey out TeX
compile endif                               --  menu for non-teX files

compile if not defined(TEX_MARGINS)
const   TEX_MARGINS = '1 70 1'              --  default margins for TeX
compile endif                               --  syntax assist

compile if not defined(TEX_TABS)
const   TEX_TABS = 8                        --  default tabs for TeX syntax
compile endif                               --  assist

compile if not defined(TEX_WANT_ACCELERATOR_KEYS)
const   TEX_WANT_ACCELERATOR_KEYS  = 0      -- default is not to add TeX
compile endif                               -- keyboard accelerators if TeX syntax
                                            -- assist is enabled

compile if not defined(TEX_ORIENTATION)
const   TEX_ORIENTATION = 'Auto'            -- default is automatic
compile endif                               -- at startup

compile if not defined(TEX_AUTO_DELETE_LOG)
const   TEX_AUTO_DELETE_LOG = 1             -- default is to turn on log file
compile endif                               -- auto-delete file at startup

compile if not defined(TEX_AUTO_SAVE_DOC)
const   TEX_AUTO_SAVE_DOC = 1               -- default is to turn on smart
compile endif                               -- save before TeX at startup

compile if not defined(TEX_DEBUG_MODE)
const   TEX_DEBUG_MODE = 0                  -- default is to turn off verbose
compile endif                               -- mode for debugging TeX
                                            -- at startup

compile if not defined(TEX_FORMAT_TO_USE)   -- default is to start with
const   TEX_FORMAT_TO_USE = 'Auto'          -- automatic format selection
compile endif

compile if not defined(TEX_USER_FILE_TYPES)
const   TEX_USER_FILE_TYPES = ''
compile endif

/*
Ŀ
  Define EPMTeX variables and post menu build                               

*/

definit
    universal tex_orient,
              tex_auto_log,
              tex_auto_save,
              tex_syntax,
              tex_debug,
              tex_format,
              tex_file_types

    tex_orient= translate(TEX_ORIENTATION)
    if tex_orient = 'PORTRAIT' then
       tex_orient = 'Portrait'
    elseif tex_orient = 'LANDSCAPE' then
           tex_orient = 'Landscape'
        elseif tex_orient = 'AUTO' then
               tex_orient = 'Auto'
            else
               sayerror 'Orientation 'TEX_ORIENTATION' not recognized. Please use Auto, Portrait, or Landscape.'
               tex_orient = 'Auto'
    endif


    tex_auto_log=   TEX_AUTO_DELETE_LOG
    tex_auto_save=  TEX_AUTO_SAVE_DOC
    tex_syntax=     TEX_SYNTAX_ASSIST
    tex_debug=      TEX_DEBUG_MODE

    tex_format=     TEX_FORMAT_TO_USE
    if translate(tex_format) = 'AUTO' then
       tex_format = 'Auto'
    endif

    tex_file_types= 'TEX STY LOG AUX '||translate(TEX_USER_FILE_TYPES)

    'PostMe BuildTexMenu'    -- delay creating menu until after the definit

/*
Ŀ
  Define TeX menu system                                                    

*/

defc BuildTexMenu
    universal  defaultmenu

--  Temporarily delete Help menu to keep it on the end of the menu bar
    deletemenu defaultmenu, HELP_MENU_ID, 0, 0

    buildsubmenu defaultmenu, 9, '~TeX', TEX_BAR__MSG, 0,0
    compile if TEX_QUICK_KEYS
        buildmenuitem defaultmenu, 9, 901, '~TeX Document'\9'Sh+F11',      'tex'TEX_TEX__MSG, 0,0
        buildmenuitem defaultmenu, 9, 924, 'Locate Next ~Error'\9'Sh+F12', 'tex_locate_error'TEX_LOCATE_ERROR__MSG, 0,0
    compile else
        buildmenuitem defaultmenu, 9, 901, '~TeX Document',      'tex'TEX_TEX__MSG, 0,0
        buildmenuitem defaultmenu, 9, 924, 'Locate Next ~Error', 'tex_locate_error'TEX_LOCATE_ERROR__MSG, 0,0
    compile endif
    buildmenuitem defaultmenu, 9, 902, \0, '', 4, 0
    buildmenuitem defaultmenu, 9, 905, 'View ~LOG File',     'tex_view_log'TEX_VIEW_LOG__MSG,  0,0
    buildmenuitem defaultmenu, 9, 903, 'Pre~View Document',  'tex_view_dvi'TEX_VIEW_DVI__MSG,  0,0
    buildmenuitem defaultmenu, 9, 904, '~Print Document',    'tex_print'TEX_PRINT__MSG,  0,0
    buildmenuitem defaultmenu, 9, 906, \0, '', 4, 0

-- Must do fancy stuff here to put menu division lines in the right
-- place regardless of user definitions

    compile if TEX_USER1_MENU_TEXT <> ''
        if TEX_USER2_MENU_TEXT <> '' then
           buildmenuitem defaultmenu, 9, 911, TEX_USER1_MENU_TEXT,'texuser1exec' TEX_USER1__MSG, 0,0
           buildmenuitem defaultmenu, 9, 921, TEX_USER2_MENU_TEXT,'texuser2exec' TEX_USER2__MSG, 0,0
           buildmenuitem defaultmenu, 9, 922, \0, '', 4, 0
        else
           buildmenuitem defaultmenu, 9, 911, TEX_USER1_MENU_TEXT,'texuser1exec' TEX_USER1__MSG, 0,0
           buildmenuitem defaultmenu, 9, 922, \0, '', 4, 0
        endif
    compile else
        if TEX_USER2_MENU_TEXT <> '' then
           buildmenuitem defaultmenu, 9, 921, TEX_USER2_MENU_TEXT,'texuser2exec' TEX_USER2__MSG, 0,0
           buildmenuitem defaultmenu, 9, 922, \0, '', 4, 0
        endif
    compile endif

    buildmenuitem defaultmenu, 9, 907, 'Delete LO~G File','tex_del_log'TEX_DEL_LOG__MSG, 0,0
    buildmenuitem defaultmenu, 9, 908, 'Delete ~DVI File','tex_del_dvi'TEX_DEL_DVI__MSG, 0,0
    buildmenuitem defaultmenu, 9, 909, \0, '', 4, 0

    buildmenuitem defaultmenu, 9, 910, '~Format', TEX_FORMAT__MSG,         17,0
    buildmenuitem defaultmenu, 9, 923, '~Automatic','texformat0'TEX_FORMAT_AUTO__MSG,0,0
    buildmenuitem defaultmenu, 9, 917, TEX_FORMAT_PLAIN_MENU_TEXT,    'texformat1'TEX_FORMAT_PLAIN__MSG,0,0

-- Must do fancy stuff here to get menu to end in the right place
-- regardless of format definitions

    compile if TEX_FORMAT_USER2_MENU_TEXT = ''
        if TEX_FORMAT_USER1_MENU_TEXT = '' then
           buildmenuitem defaultmenu, 9, 918, TEX_FORMAT_LATEX_MENU_TEXT,'texformat2'TEX_FORMAT_LATEX__MSG,32768+1,0
        else
           buildmenuitem defaultmenu, 9, 918, TEX_FORMAT_LATEX_MENU_TEXT,'texformat2'TEX_FORMAT_LATEX__MSG,0,0
           buildmenuitem defaultmenu, 9, 919, TEX_FORMAT_USER1_MENU_TEXT,'texformat3'TEX_FORMAT_USER1__MSG,32768+1,0
        endif
    compile else
        if TEX_FORMAT_USER1_MENU_TEXT = '' then
           buildmenuitem defaultmenu, 9, 918, TEX_FORMAT_LATEX_MENU_TEXT,'texformat2'TEX_FORMAT_LATEX__MSG,0,0
           buildmenuitem defaultmenu, 9, 920, TEX_FORMAT_USER2_MENU_TEXT,'texformat4'TEX_FORMAT_USER2__MSG,32768+1,0
        else
           buildmenuitem defaultmenu, 9, 918, TEX_FORMAT_LATEX_MENU_TEXT,'texformat2'TEX_FORMAT_LATEX__MSG,0,0
           buildmenuitem defaultmenu, 9, 919, TEX_FORMAT_USER1_MENU_TEXT,'texformat3'TEX_FORMAT_USER1__MSG,0,0
           buildmenuitem defaultmenu, 9, 920, TEX_FORMAT_USER2_MENU_TEXT,'texformat4'TEX_FORMAT_USER2__MSG,32768+1,0
        endif
    compile endif

    buildmenuitem defaultmenu, 9, 912, '~Orientation', TEX_ORIENT__MSG,17,0
    buildmenuitem defaultmenu, 9, 925, '~Automatic','texorient0'TEX_ORIENT_AUTO__MSG,0,0
    buildmenuitem defaultmenu, 9, 926, '~Portrait', 'texorient1'TEX_ORIENT_PORT__MSG,0,0
    buildmenuitem defaultmenu, 9, 927, '~Landscape','texorient2'TEX_ORIENT_LAND__MSG,32768+1,0

    buildmenuitem defaultmenu, 9, 913, '~Auto LOG Delete','toggle_auto_log'TEX_AUTO_DEL_LOG__MSG, 0,0
    buildmenuitem defaultmenu, 9, 914, 'Auto ~Save','toggle_auto_save'TEX_AUTO_SAVE__MSG, 0,0
    compile if TEX_SYNTAX_ASSIST
        buildmenuitem defaultmenu, 9, 915, 'Synta~x Expansion','toggle_tex_expand'TEX_SYNTAX__MSG, 0,0
    compile endif
    buildmenuitem defaultmenu, 9, 916, 'De~bug mode','toggle_debug'TEX_DEBUG__MSG, 0,0
    buildmenuitem defaultmenu, 9, 928, \0, '', 4, 0
    buildmenuitem defaultmenu, 9, 929, 'A~bout...','tex_about_box'TEX_ABOUT_BOX__MSG, 0,0

    call readd_help_menu();   -- Put Help menu back in

    showmenu defaultmenu

   'PostMe  TexMenuEnable'

-----------------------------------------------------------------------------
-- enable the TeX menu only if TeX file loaded
-- disable if TEX_MENU_ALWAYS_ON is set

defc TexMenuEnable
    universal  tex_debug,
               tex_file_types,
               default_font

    compile if TEX_MENU_ALWAYS_ON = 1
       enabled = 1
    compile else
       ext = filetype()
       enabled = pos(leftstr(ext,3),tex_file_types)
    compile endif

--  Change working directory to directory of current file since that is
--  where the .DVI and .LOG files (and any others) should go.
    if enabled then
        n=lastpos('\',.filename)
        dir = substr(.filename,1,n)
--        if tex_debug then sayerror 'Changing directory to 'dir; endif
        call directory(leftstr(dir,length(dir)-1))
    endif

    SetMenuAttribute(9, 16384, enabled)

/*
Ŀ
  Command definitions                                                       

*/
-----------------------------------------------------------------------------
-- Show the about box

defc tex_about_box

-- Text is limited to 256 chars!  So we can't say as much as we'd like
-- There are 253 chars currently in message (I think)

   call WinMessageBox('About EPMTeX 2.0',

--  Can't fit all of this
--                      'This is the beta version of'\13||
--                      'EPMTeX 2.0 by Jonathan Hacker'\13||
--                      'and Rodney Korte.'\13||

                      'EPMTeX 2.0'\13\13||
--  Don't like this
--                      'by'\13||
--                      '       Jonathan Hacker'\13||
--                      '&     Rodney Korte.'\13\13||
                      'by    Jonathan Hacker'\13||
                      '&       Rodney Korte'\13\13||
                      'EPMTeX is governed by the'\13||
                      'license agreement included'\13||
                      'with the EPMTeX package.'\13\13||

--  Can't fit all of this
--                      'Send questions, comments, or problems to:'\13||

                      'Questions, comments, problems:'\13||
                      '       korte@sabine.psu.edu'\13||
                      'or    hacker@patagonia.bellcore.com'\13\13||
                      'June 1994',
                      MB_OK+MB_INFORMATION+MB_DEFBUTTON1+MB_MOVEABLE)

-----------------------------------------------------------------------------
-- Create an OS/2 window & execute TeX in it.

defc tex
    universal  tex_auto_save,
               tex_debug,
               tex_format,
               tex_log_file_line

    mouse_setpointer WAIT_POINTER

    master = tex_master_name('TeX Document')

-- Need to change to master file to get Format from first line
-- And TeX in that directory

    if master = '-1' then
       mouse_setpointer EPM_POINTER
       RETURN
    endif

    if master = '' then
        call tex_parse_filename(.filename,dr,dir,stem,ext)
        call tex_check_for_save()
        working_file = ''
    else
        working_file = .filename
        call tex_parse_filename(master,dr,dir,stem,ext)
        if tex_file_in_ring(master) then
            'edit /n' master
            call tex_check_for_save()
        else
            'edit /d' master
        endif
    endif

    for counter = 1 to .last
        getline line, counter
        parse value line with '\input' input_fname .
        input_fname = strip(input_fname)
-- Translate from unix-style to OS/2-style
        input_fname = translate(input_fname,'\','/')
        if pos('.',input_fname) = 0 then
            input_fname = input_fname||'.tex'
        endif
        if tex_file_in_ring(input_fname) = 1 then
            current_file = .filename
            'edit /n' input_fname
            call tex_check_for_save()
            'edit /n' current_file
        endif
    endfor

    if tex_format = 'Auto' then
        format = tex_auto_format()
    else
        sayerror 'Using the 'tex_format' format as selected.'
        format = tex_format
    endif

--  Must use unix-style filename since the filename is fully-qualified,
--  and tex will interpret the backslashes as the start of a control
--  sequence.

    unixfilename = translate(dr||dir||stem,'/','\')
    command=TEX_EXEC||format' 'unixfilename
    isCMD = TEX_IS_CMD

    if tex_debug then sayerror command; endif

-- We are not getting the correct directory here, even though it should be
-- handled by TexMenuEnable?  This results in .log file being placed in
-- the wrong directory in some cases- the .log file should be in the same
-- directory as the master file.

--  Change working directory to directory of current file since that is
--  where the .DVI and .LOG files (and any others) should go.

    savedir = directory()    -- save directory **

    tstr = dr || leftstr(dir,length(dir)-1)
    if not (savedir = tstr) then
--        if tex_debug then sayerror 'Changing directory to 'tstr; endif
        call directory(tstr)     -- change directory **
    endif

    if isCMD then            -- command is a batch file, use command processor
        'start /c /win 'command
    else
        'start /n /win 'command
    endif
    if rc <> 0 then
        sayerror"Caution! Could not start TeX. Exited with return code = "rc
    else

-- Remove old LOG file from the ring after successful TeX compilation
        logfile = stem'.log'
        if tex_file_in_ring(logfile) = 1 then
            'edit /n' logfile       -- select logfile in ring
            .modify = 0             -- set this in case user modified file
            'quit'                  -- close the file
            if tex_debug then sayerror 'Old log file removed from ring'; endif
        endif
--      Can't load new log file here unless we wait for TeX to finish
        tex_log_file_line = ''
    endif

    if not (savedir = tstr) then
        call directory(savedir)  -- restore directory **
    endif

-- Change back to file we from which we called TeX
    if not (working_file = '') then
        'edit /n' working_file
    endif

    mouse_setpointer EPM_POINTER

------------------------------------------------------------------------------
-- View current TeX file in previewer.

defc tex_view_dvi
    universal  tex_orient,
               tex_auto_log,
               tex_debug

    mouse_setpointer WAIT_POINTER

    master = tex_master_name('Preview Document')

    if master = '-1' then
       mouse_setpointer EPM_POINTER
       RETURN
    endif

    if master = '' then
        call tex_parse_filename(.filename,dr,dir,stem,ext)
    else
        call tex_parse_filename(master,dr,dir,stem,ext)
    endif

    if  tex_auto_log and exist(dr||dir||stem'.log') then
        call tex_delete_log_file()
    endif

    dvifile = dr||dir||stem||'.dvi'

    if tex_orient = 'Auto' then
        orientation = tex_auto_orient()
    else
        sayerror 'Viewing in 'tex_orient' orientation'
        orientation = tex_orient
    endif

    if orientation = 'Landscape' then
        command= TEX_VIEWLA_EXEC' 'dvifile
        isCMD = TEX_VIEWLA_IS_CMD
    else
        command= TEX_VIEW_EXEC' 'dvifile
        isCMD = TEX_VIEW_IS_CMD
    endif

    if tex_debug then sayerror command; endif

--  Change working directory to directory of current file since that is
--  where the .DVI and .LOG files (and any others) should go.

    savedir = directory()    -- save directory **

    tstr = dr || leftstr(dir,length(dir)-1)
    if not (savedir = tstr) then
--        if tex_debug then sayerror 'Changing directory to 'tstr; endif
        call directory(tstr)     -- change directory **
    endif

-- NOTE: add /fs or /win if previewer is not a PM app

    if isCMD then            -- command is a batch file, use command processor
        'start /c 'command
    else
        'start /n 'command
    endif

    if rc <> 0 then
        sayerror "Caution! Could not start Previewer. Exited with return code = "rc
    endif

    if not (savedir = tstr) then
        call directory(savedir)  -- restore directory **
    endif

    mouse_setpointer EPM_POINTER

-----------------------------------------------------------------------------
-- Create an OS/2 window & execute TeX printer driver.

defc tex_print
    universal  tex_orient,
               tex_auto_log,
               tex_debug

    mouse_setpointer WAIT_POINTER

    master = tex_master_name('Print Document')

    if master = '-1' then
       mouse_setpointer EPM_POINTER
       RETURN
    endif

    if master = '' then
        call tex_parse_filename(.filename,dr,dir,stem,ext)
    else
        call tex_parse_filename(master,dr,dir,stem,ext)
    endif

    if  tex_auto_log and exist(dr||dir||stem'.log') then
        call tex_delete_log_file()
    endif

    dvifile = dr||dir||stem||'.dvi'

    if tex_orient = 'Auto' then
        orientation = tex_auto_orient()
    else
        sayerror 'Printing in 'tex_orient' orientation'
        orientation = tex_orient
    endif

    if orientation = 'Landscape' then
        command= TEX_PRINTLA_EXEC' 'dvifile
        isCMD = TEX_PRINTLA_IS_CMD
    else
        command= TEX_PRINT_EXEC' 'dvifile
        isCMD = TEX_PRINT_IS_CMD
    endif

    if tex_debug then sayerror command; endif

--  Change working directory to directory of current file since that is
--  where the .DVI and .LOG files (and any others) should go.

    savedir = directory()    -- save directory **

    tstr = dr || leftstr(dir,length(dir)-1)
    if not (savedir = tstr) then
--        if tex_debug then sayerror 'Changing directory to 'tstr; endif
        call directory(tstr)     -- change directory **
    endif

    if isCMD then            -- command is a batch file, use command processor
        'start /c /win 'command
    else
        'start /n /win 'command
    endif
    if rc <> 0 then
        sayerror "Caution! Could not start Printer driver. Exited with return code = "rc
    endif

    if not (savedir = tstr) then
        call directory(savedir)  -- restore directory **
    endif

    mouse_setpointer EPM_POINTER

-----------------------------------------------------------------------------
-- Load LOG file for current TeX file (or associated master file) if
-- it exists.

defc tex_view_log
    universal  tex_log_file_line,
               tex_log_file_last,
               tex_error_fname,
               name_count,
               par_count

    mouse_setpointer WAIT_POINTER

    master = tex_master_name('View LOG File')

    if master = '-1' then
       mouse_setpointer EPM_POINTER
       RETURN
    endif

    if master = '' then
        call tex_parse_filename(.filename,dr,dir,stem,ext)
    else
        call tex_parse_filename(master,dr,dir,stem,ext)
    endif

-- Get rid of old copy in ring if it exists
    logfile = dr||dir||stem||'.log'
    if tex_file_in_ring(logfile) = 1 then
        'edit /n' logfile                     -- select logfile in ring
        .modify = 0                           -- set this in case modified
        'quit'                                -- close the file
    endif

-- This file exists, otherwise we couldn't choose this command from
-- the menu.
    'edit /d' logfile

    tex_log_file_line  = 3
    tex_log_file_last = .last
    tex_error_fname = dr||dir||stem||'.tex'
    name_count = 0
    par_count = 1

    mouse_setpointer EPM_POINTER

-----------------------------------------------------------------------------
-- Create an OS/2 window & execute user defined command 1.

defc texuser1exec
    universal  tex_orient,
               tex_debug

    mouse_setpointer WAIT_POINTER

    master = tex_master_name(TEX_USER1_MENU_TEXT)

    if master = '-1' then
       mouse_setpointer EPM_POINTER
       RETURN
    endif

    if master = '' then
        call tex_parse_filename(.filename,dr,dir,stem,ext)
    else
        call tex_parse_filename(master,dr,dir,stem,ext)
    endif

    fname = dr||dir||stem||TEX_USER1_EXT
--  If this command runs TeX, the next line needs to be uncommented!
--    fname = translate(fname,'/','\')

    if tex_orient = 'Auto' then
        orientation = tex_auto_orient()
    else
        sayerror 'Doing User defined command #1 in 'tex_orient' orientation'
        orientation = tex_orient
    endif

    if orientation = 'Landscape' then
        command= TEX_USER1LA_EXEC' 'fname
        isCMD = TEX_USER1LA_IS_CMD
    else
        command= TEX_USER1_EXEC' 'fname
        isCMD = TEX_USER1_IS_CMD
    endif

    if tex_debug then sayerror command; endif

--  Change working directory to directory of current file since that is
--  where the .DVI and .LOG files (and any others) should go.

    savedir = directory()    -- save directory **

    tstr = dr || leftstr(dir,length(dir)-1)
    if not (savedir = tstr) then
--        if tex_debug then sayerror 'Changing directory to 'tstr; endif
        call directory(tstr)     -- change directory **
    endif

    if isCMD then            -- command is a batch file, use command processor
        'start /c /win 'command
    else
        'start /n /win 'command
    endif

    if rc <> 0 then
        sayerror "Caution! Could not execute user-defined command 1. Exited with return code = "rc
    endif

    if not (savedir = tstr) then
        call directory(savedir)  -- restore directory **
    endif

    mouse_setpointer EPM_POINTER

-----------------------------------------------------------------------------
-- Create an OS/2 window & execute user defined command 2.

defc texuser2exec
    universal  tex_orient,
               tex_debug

    mouse_setpointer WAIT_POINTER

    master = tex_master_name(TEX_USER2_MENU_TEXT)

    if master = '-1' then
       mouse_setpointer EPM_POINTER
       RETURN
    endif

    if master = '' then
        call tex_parse_filename(.filename,dr,dir,stem,ext)
    else
        call tex_parse_filename(master,dr,dir,stem,ext)
    endif

    fname = dr||dir||stem||TEX_USER2_EXT
--  If this command runs TeX, the next line needs to be uncommented!
--    fname = translate(fname,'/','\')

    if tex_orient = 'Auto' then
        orientation = tex_auto_orient()
    else
        sayerror 'Doing User defined command #1 in 'tex_orient' orientation'
        orientation = tex_orient
    endif

    if orientation = 'Landscape' then
        command= TEX_USER2LA_EXEC' 'fname
        isCMD = TEX_USER2LA_IS_CMD
    else
        command= TEX_USER2_EXEC' 'fname
        isCMD = TEX_USER2_IS_CMD
    endif

    if tex_debug then sayerror command; endif

--  Change working directory to directory of current file since that is
--  where the .DVI and .LOG files (and any others) should go.

    savedir = directory()    -- save directory **

    tstr = dr || leftstr(dir,length(dir)-1)
    if not (savedir = tstr) then
--        if tex_debug then sayerror 'Changing directory to 'tstr; endif
        call directory(tstr)     -- change directory **
    endif

    if isCMD then            -- command is a batch file, use command processor
        'start /c /win 'command
    else
        'start /n /win 'command
    endif

    if rc <> 0 then
        sayerror "Caution! Could not execute user-defined command 2. Exited with return code = "rc
    endif

    if not (savedir = tstr) then
        call directory(savedir)  -- restore directory **
    endif

    mouse_setpointer EPM_POINTER

-----------------------------------------------------------------------------
-- Read LOG file and find errors in source code

defc tex_locate_error
    call tex_find_errors()

-----------------------------------------------------------------------------
-- Delete LOG file associated with the master file for the current
-- file, if it exists, or the LOG file for the current file.

defc tex_del_log
    call tex_delete_log_file()

-----------------------------------------------------------------------------
-- Delete LOG file associated with the master file for the current
-- file, if it exists, or the LOG file for the current file.

defc tex_del_dvi
    call tex_delete_dvi_file()

-----------------------------------------------------------------------------
-- Calls to change formats

defc texformat0   call set_tex_format(0)
defc texformat1   call set_tex_format(1)
defc texformat2   call set_tex_format(2)
defc texformat3   call set_tex_format(3)
defc texformat4   call set_tex_format(4)

-----------------------------------------------------------------------------
-- Calls to change orientation

defc texorient0   call set_tex_orient(0)
defc texorient1   call set_tex_orient(1)
defc texorient2   call set_tex_orient(2)

-----------------------------------------------------------------------------
-- Toggle Landscape mode flag.
--
--defc toggle_landscape
--    universal  tex_landscape,
--              tex_debug
--
--    tex_landscape = not tex_landscape
--    if tex_debug then sayerror 'tex_landscape = 'tex_landscape; endif
--
-----------------------------------------------------------------------------
-- Toggle Auto-Delete current TeX LOG file after View or Print flag.

defc toggle_auto_log
    universal  tex_auto_log,
               tex_debug

    tex_auto_log = not tex_auto_log
    if tex_debug then sayerror 'tex_auto_log = 'tex_auto_log; endif

-----------------------------------------------------------------------------
-- Toggle Auto smart-save

defc toggle_auto_save
    universal  tex_auto_save,
               tex_debug

    tex_auto_save = not tex_auto_save
    if tex_debug then sayerror 'tex_auto_save = 'tex_auto_save; endif

-----------------------------------------------------------------------------
-- Toggle EPM syntax expansion flag.

compile if TEX_SYNTAX_ASSIST
  defc toggle_tex_expand
    universal  expand_on,
               tex_debug

    if expand_on then
      'expand 'OFF__MSG
    else
      'expand 'ON__MSG
    endif
    if tex_debug then sayerror 'expand_on = 'expand_on; endif
compile endif

-----------------------------------------------------------------------------
-- Toggle debug mode for troubleshooting TeX commands.

defc toggle_debug
    universal  tex_debug

    tex_debug = not tex_debug
    if tex_debug then sayerror 'tex_debug = 'tex_debug; endif

/*
Ŀ
  Procedure definitions                                                     

*/
-----------------------------------------------------------------------------
-- Define TeX format

defproc set_tex_format =
    universal  tex_debug,
               tex_format

    if arg(1) = 0 then tex_format = 'Auto'
    elseif arg(1) = 2 then tex_format = TEX_FORMAT_LATEX
        elseif arg(1) = 3 and TEX_FORMAT_USER1_MENU_TEXT <> '' then tex_format = TEX_FORMAT_USER1
            elseif arg(1) = 4 and TEX_FORMAT_USER2_MENU_TEXT <> '' then tex_format = TEX_FORMAT_USER2
                else tex_format = TEX_FORMAT_PLAIN
    endif
--    if tex_debug then sayerror 'tex_format('arg(1)') = 'tex_format; endif

-----------------------------------------------------------------------------
-- Define TeX orientation

defproc set_tex_orient =
    universal  tex_debug,
               tex_orient

    if arg(1) = 0 then tex_orient = 'Auto'
    elseif arg(1) = 2 then tex_orient = 'Landscape'
        else tex_orient = 'Portrait'
    endif

--    if tex_debug then sayerror 'tex_orient('arg(1)') = 'tex_orient; endif

-----------------------------------------------------------------------------
-- Determine format to use with this file from first line the file.
-- Similar specification as Eberhard Mattes' TEXIT.CMD distributed with emTeX.

defproc tex_auto_orient =
    universal  tex_debug

    orientation = ''
    getline line, 1
    line = translate(line)
    parse value line with '%' 'ORIENTATION' ':' orientation .

    if orientation = '' then
        sayerror 'Auto orientation select: No orientation specified.  Portrait will be used.'
        return 'Portrait'
    elseif orientation = 'PORTRAIT' then
            sayerror 'Auto orientation select: Portrait orientation will be used.'
            return 'Portrait'
        elseif orientation = 'LANDSCAPE' then
            sayerror 'Auto orientation select: Landscape orientation will be used.'
            return 'Landscape'
        else
          sayerror 'Auto orientation select: Orientation 'orientation' not recognized.  Portrait will be used.'
          return 'Portrait'
    endif

-----------------------------------------------------------------------------
-- Determine format to use with this file from first line the file.
-- Same specification as Eberhard Mattes' TEXIT.CMD distributed with emTeX.

defproc tex_auto_format =
    universal  tex_debug

    format = ''
    getline line, 1
    parse value line with '%' . 'Format' ':' format .
    if format = '' then
        parse value line with '%' . 'format' ':' format .
    endif
    if format = '' then
        parse value line with '%' . 'Format' format .
    endif
    if format = '' then
        parse value line with '%' . 'format' format .
    endif

    if format = '' then
        sayerror 'Auto format select: No format specified.  Plain will be used.'
        return TEX_FORMAT_PLAIN
    else
        sayerror 'Auto format select: The format 'format' will be used.'
        return format
    endif

-----------------------------------------------------------------------------
-- called by mymnuini.e to get which format is chosen
-- called with negative argument to check existence, called with
-- positive argument to see if this is the one

defproc get_tex_format =
    universal  tex_format,
               tex_debug

    if arg(1) < 0 then
        if arg(1) = -3 and TEX_FORMAT_USER1_MENU_TEXT <> '' then return 1
        elseif arg(1) = -4 and TEX_FORMAT_USER2_MENU_TEXT <> '' then return 1
            else return 0
        endif
    else
        if  (arg(1) = 0 and tex_format ='Auto')         or
            (arg(1) = 1 and translate(tex_format) = translate(TEX_FORMAT_PLAIN))  or
            (arg(1) = 2 and translate(tex_format) = translate(TEX_FORMAT_LATEX))  or
            (arg(1) = 3 and TEX_FORMAT_USER1 <> '' and translate(tex_format) = translate(TEX_FORMAT_USER1)) or
            (arg(1) = 4 and TEX_FORMAT_USER2 <> '' and translate(tex_format) = translate(TEX_FORMAT_USER2)) then
            return 1
        else
            return 0
        endif
    endif

-----------------------------------------------------------------------------
-- Delete LOG file associated with the master file for the current
-- file, if it exists, or the LOG file for the current file.

defproc tex_delete_log_file
    universal  tex_log_file_line

    call tex_parse_filename(.filename,dr,dir,stem,ext)

    master = tex_master_name('Delete LOG File')

    if master = '-1' then
       mouse_setpointer EPM_POINTER
       RETURN
    endif

    if not ( master = '') then
        call tex_parse_filename(master,dr,dir,stem,ext)
    endif

--  Remove log file from ring also if it is there

    logfile = dr||dir||stem||'.log'
    if tex_file_in_ring(logfile) = 1 then
        'edit /n' logfile                     -- select logfile in ring
        .modify = 0                           -- set this in case modified
        'quit'                                -- close the file
    endif

    rc = erasetemp(logfile)
    if rc = 0 then
        sayerror 'File 'logfile' deleted'
    elseif rc = 2 then
        sayerror 'Delete LOG: file 'logfile' could not be found'
    else
        sayerror 'Delete LOG: file 'logfile' could not be deleted'
        beep(1000, 100)
    endif

    tex_log_file_line = ''

-----------------------------------------------------------------------------
-- Delete DVI file associated with the master file for the current
-- file, if it exists, or the DVI file for the current file.

defproc tex_delete_dvi_file

    call tex_parse_filename(.filename,dr,dir,stem,ext)

    master = tex_master_name('Delete DVI File')

    if master = '-1' then
       mouse_setpointer EPM_POINTER
       RETURN
    endif

    if not ( master = '') then
        call tex_parse_filename(master,dr,dir,stem,ext)
    endif

    dvifile = dr||dir||stem||'.dvi'

    rc = erasetemp(dvifile)
    if rc = 0 then
         sayerror 'File 'dvifile' deleted'
    elseif rc = 2 then
         sayerror 'Delete DVI: file 'dvifile' could not be found'
    else
         sayerror 'Delete DVI: file 'dvifile' could not be deleted'
         beep(1000, 100)
    endif

-----------------------------------------------------------------------------
-- called in mymnuini.e to get state of global TeX variable tex_orient
-- called with positive argument to see if this is the one

defproc get_tex_orient =
    universal  tex_orient,
               tex_debug

    if  (arg(1) = 0 and tex_orient ='Auto')        or
        (arg(1) = 1 and tex_orient = 'Portrait')   or
        (arg(1) = 2 and tex_orient = 'Landscape')  then
        return 1
    else
        return 0
    endif

-----------------------------------------------------------------------------
-- called in mymnuini.e to get state of global TeX variable tex_auto_log

defproc get_tex_auto_log =
    universal  tex_auto_log

    return tex_auto_log

-----------------------------------------------------------------------------
-- called in mymnuini.e to get state of global TeX variable tex_auto_save

defproc get_tex_auto_save =
    universal  tex_auto_save

    return tex_auto_save

-----------------------------------------------------------------------------
-- called in mymnuini.e to get info on syntax menu options
--  arg = 0 -> query tex_syntax flag
--        1 -> query  expand_on flag

defproc get_tex_syntax =
    universal  tex_syntax,
               expand_on

    if arg(1) = 0 then           -- query tex_syntax
        return tex_syntax
    else
        return expand_on         -- query expand_on
    endif

-----------------------------------------------------------------------------
-- called in mymnuini.e to get state of global EPM variable expand_on

defproc get_tex_expand_on =
    universal  expand_on

    return expand_on

-----------------------------------------------------------------------------
-- called in mymnuini.e to get state of global TeX variable tex_debug

defproc get_tex_debug =
    universal  tex_debug

    return tex_debug

-----------------------------------------------------------------------------
-- Called in mymnuini.e to determine if DVI or LOG file exists for
-- associated master file if it exists, or for current file.
-- Returns 1 if file does not exist, 0 if it does

defproc tex_check_exist =
    universal  tex_debug

    master = tex_master_name()
    if master = '' then
        fname = .filename
    else
        fname = master
    endif

    call tex_parse_filename(fname,dr,dir,stem,ext)

    return(not exist(dr||dir||stem||arg(1)))

-----------------------------------------------------------------------------
-- called in mymnuini.e to get the value of TEX_USER?_EXT

defproc get_tex_user_ext =

    if arg(1) = 1 then return TEX_USER1_EXT
    elseif arg(1) = 2 then return TEX_USER2_EXT
        else return ''
    endif

-----------------------------------------------------------------------------
-- called in mymnuini.e to determine if TEX_USER?_EXEC is defined

defproc tex_check_user_exec =

    if (arg(1) = 1 and TEX_USER1_EXEC <> '')  OR
       (arg(1) = 2 and TEX_USER2_EXEC <> '') then
        return 1
    else
        return 0
    endif

-----------------------------------------------------------------------------
--  Check to see if current file has been modified.  If it has, save
--  it if auto-save is enabled, otherwise as user about it.

defproc tex_check_for_save=
    universal  tex_auto_save

    if .modify then
        if tex_auto_save then
           'Save'
            sayerror 'Saving modified document '.filename' before calling TeX'
            refresh
            delay(10)
        else
            refresh
            if AskYesNo('File '.filename' modified. Save?',1) = 'Y' then
                'Save'
                sayerror 'Saving modified document '.filename
            else
                sayerror 'Will TeX with stale document '.filename
                beep(1000,100)
            endif
        endif
    endif

-----------------------------------------------------------------------------
--  Called with optional argument which should be the Menu item
--    associated with the calling routine.  This is used as caption for
--    message box announcing that specified master file doesn't exist.
--  Extract master file name from first line of file
--  Returns: a) '' if master file is not specified
--           b) master file name with path added on if file exists
--           c) '-1' if arg(1) <> '' AND master file is specified
--                 but doesn't exist.  Got it?

defproc tex_master_name
    universal  tex_debug

    master = ''
    getline line, 1
    parse value line with '%' . 'Master' ':' master .
    if master = '' then
        parse value line with '%' . 'master' ':' master .
    endif
    if master = '' then
        parse value line with '%' . 'Master' master .
    endif
    if master = '' then
        parse value line with '%' . 'master' master .
    endif

    if tex_debug then
        if master = '' then
            sayerror 'Master file not defined.'
        else
            sayerror 'Master file found to be 'master' from directive.'
        endif
    endif

    if not (master = '') then
        tex_parse_filename(.filename,dr,dir,stem,ext)
        master = translate(master,'\','/')
        master = master||'.tex'

--  Now, make the master filename fully qualified
--      If only a stem.ext
        if lastpos('\',master) = 0 then
            master = dr||dir||master
        else
--          if no drive letter
            if leftstr(master,1) = '\' then
                master = dr||master
            endif
        endif

        if not exist(master) then
--           sayerror 'Caution! Master file 'master' not found!'
--           master = ''
           if not (arg(1) = '') then
               rc = WinMessageBox(arg(1),
                                  'The specified master file'\13||
                                  master\13||
                                  'does not exist.'\13||
                                  ''\13||
                                  'Continue processing current file'\13||
                                  'as stand-alone?'\13||
                                  ''\13,
                                  MB_YESNO+MB_WARNING+MB_DEFBUTTON2+MB_MOVEABLE)
               if rc = MBID_NO then
                   master = '-1'
               else
                   master = ''
               endif
           else
               master = ''
           endif
        endif
    endif

    return master

-----------------------------------------------------------------------------
-- Parse a file name:  d:\dir1\dir2\file.ext
--                     \dir1\dir2\file.ext
--                     file.ext
--                     .ext
--                     \dir1\.ext
--                     ...etc,  but NOT d:test2.ext
-- returns  dr      d:
--          dir     \dir1\dir2\    if any of these does not exist,
--          stem    file           a null is returned.
--          ext     ext

defproc tex_parse_filename(fname, var dr, var dir, var stem, var ext)

-- Translate from Unix-style to OS/2-style names
    fname = translate(fname,'\','/')
    parse value fname with  tmp '.' ext
    n = lastpos('\',tmp)
    stem = substr(tmp,n+1)
    tmp = substr(tmp,1,n)
    n = pos(':',tmp)
    dr = substr(tmp,1,n)
    dir = substr(tmp,n+1)

-----------------------------------------------------------------------------
-- Determine if a file already is in ring.  Decision based on filename
-- only without path, since pfile_exists() doesn't seem to like path info.

defproc tex_file_in_ring(fname)

    tex_parse_filename(fname,dr,dir,stem,ext)

    return pfile_exists(stem||'.'||ext)

-----------------------------------------------------------------------------
-- Called my mymnuini.e to determine whether or not to disable
-- 'Locate Next Error' menu choice when log file exists
-- Returns 1 if there are more errors, 0 if not.

defproc tex_more_errors
    universal  tex_log_file_line,
               tex_log_file_last

-- First expression is true when there is no log file in ring.
-- Second expression true when there is, but haven't gotten through
-- all errors yet.

    return ( (tex_log_file_line = '') or
            (tex_log_file_line < tex_log_file_last) )

-----------------------------------------------------------------------------
--  Read LOG file and find errors in source code

defproc tex_find_errors
    universal  tex_log_file_line,
               tex_log_file_last,
               tex_error_fname,
               tex_return_to_file,
               tex_file_types,
               tex_debug,
               name_count,
               par_count,
               EPM_utility_array_ID

    mouse_setpointer WAIT_POINTER

    empty = ''

    tex_return_to_file = .filename

    master = tex_master_name('Locate Next Error')

    if master = '-1' then
       mouse_setpointer EPM_POINTER
       RETURN
    endif

    if master = '' then
        fname = .filename
    else
        fname = master
    endif

    tex_parse_filename(fname,dr,dir,stem,ext)

--  The log file exists, or we would not be able to choose this routine
--  from the menu.
    logfile = dr||dir||stem||'.LOG'
    if tex_file_in_ring(logfile) = 1 then
        'edit /n' logfile
    else
        'edit /d' logfile
        tex_log_file_line  = 3
        tex_log_file_last = .last
        tex_error_fname = dr||dir||stem||'.tex'
        name_count = 0
        par_count = 1
    endif

    do while (tex_log_file_line < tex_log_file_last)

      getline line, tex_log_file_line

      if leftstr(line,1) = '!' then
          parse value line with '!'error_msg
          do while not (leftstr(line,1) = 'l')
              tex_log_file_line = tex_log_file_line + 1
              getline line, tex_log_file_line
          enddo
          parse value line with 'l.' linenum contexttext
          sayerror 'TeX error msg -> 'error_msg':   'contexttext
          if tex_file_in_ring(tex_error_fname) = 1 then
              if tex_debug then sayerror 'Editing 'tex_error_fname' from ring.'; endif
              'edit /n' tex_error_fname
          else
-- This file should exist since we 'just' compiled it.
              if tex_debug then sayerror 'Editing 'tex_error_fname' from disk.'; endif
              'edit /d' tex_error_fname
          endif
--  Check to see if this could be the right file. We don't want to bomb out
--  trying to getline after the end of the file.
          if linenum > .last then
             sayerror 'Something is amiss! I''m lost. Apparently the next error is not in this file!'
          else
              getline errorline, linenum
              tex_log_file_line = tex_log_file_line + 1
              if length(contexttext) > 4 then
                  chopum = 4
              else
                  chopum = 1
              endif
              col1 = pos( substr(contexttext,chopum), errorline)
              col2 = col1 + length(contexttext) - chopum + 1
              .line = linenum
              .col  = col2
-- For some reason, the circle disappears half the time!
-- Apparently, there is a refresh or something after the circle.
-- Putting in the refresh below didn't help at all.  Ideas?
--              refresh
              circleit 2,linenum,col1,col2,1
          endif
          mouse_setpointer EPM_POINTER
          RETURN
      else    /* This is not a line starting with '!' */

          do while not ( line = '')

--              if tex_debug then sayerror 'line='line; endif
              left_par =  pos('(',line)
              right_par = pos(')',line)
--              if tex_debug then sayerror 'left_par='left_par'  right_par='right_par; endif
-- EPM can only handle lines of 255 chars anyway, so make them 256 if the
-- are 0 to simplify things a bit.
              if left_par = 0 then left_par = 256 endif
              if right_par = 0 then right_par = 256 endif

              if (right_par < left_par) then
                  par_count = par_count - 1
--                  if tex_debug then sayerror 'par_count (decreased) ='par_count; endif
                  if par_count = 0 then
--                      if tex_debug then sayerror 'Erasing name 'name_count' from array'; endif
                      do_array 2, EPM_utility_array_ID, 'tex_fname.'name_count, empty
                      do_array 2, EPM_utility_array_ID, 'tex_par.'name_count, empty
                      name_count = name_count - 1
-- name_count will be 0 when we are finished
                      if name_count > 0 then
                          do_array 3, EPM_utility_array_ID, 'tex_fname.'name_count, tex_error_fname
                          do_array 3, EPM_utility_array_ID, 'tex_par.'name_count, par_count
--                          if tex_debug then sayerror 'Got 'tex_error_fname'('name_count') and par_count='par_count; endif
                      else
--                          if tex_debug then sayerror 'name_count = 0'; endif
                      endif
                  endif
--                  if tex_debug then sayerror 'line=   'line; endif
                  line = substr(line,right_par+1)
--                  if tex_debug then sayerror 'newline='line; endif
              else
                  if not (left_par = 256) then
                      line = substr(line,left_par+1)
                      if tex_extract_fname(line,dr,dir) = 0 then
                          par_count = par_count + 1
--                          if tex_debug then sayerror 'par_count (increased) ='par_count; endif
                      else
                          par_count = 1
--                          if tex_debug then sayerror 'par_count set to 1'; endif
                      endif
                  else
                      line = ''
                  endif
              endif
          enddo  /*  while not (line='')  */
          tex_log_file_line = tex_log_file_line + 1
      endif

    enddo     /*  while (tex_log_file_line < tex_log_file_last)  */

    if not (tex_log_file_line < tex_log_file_last) then
        sayerror 'No more errors'
        'edit /n' tex_return_to_file
    endif

    mouse_setpointer EPM_POINTER

-----------------------------------------------------------------------------
-- This routine called by tex_find_errors after a left parenth is found
-- to determine whether or not the text following the parenth is a filename.
-- If it is, tex_error_fname is changed to that name, and the routine
-- returns 1.  Otherwise, a 0 is returned.
--
-- The logic goes like this:
--  1) make sure it has a '.' in it
--  2) parse it into pieces before '.' and after '.'
--  3) make sure that the part after has at least 3 characters
--  4) make sure that the part after is one of the defined tex filetypes
--  5) if the above is true the name is fully qualified and placed in
--     tex_error_fname, and is also put into an array.
--  6) take name out of "line"
--  7) return

defproc tex_extract_fname(var line, dr, dir)
    universal  tex_debug,
               tex_error_fname,
               tex_file_types,
               par_count,
               name_count,
               EPM_utility_array_ID

    flag = 0

-- If there is no '.' in the line, there cannot be a filename in it
    if not (pos('.',line) = 0) then
        parse value line with tmp1 '.' tmp2
--        if tex_debug then sayerror 'tmp1='tmp1'  tmp2='tmp2; endif
-- If the part after the '.' is less than 3 chars long, it is not a filename
        if (length(tmp2) > 2) then
--          Take first 3 chars after '.' and turn into all uppercase
            tmp2 = translate(leftstr(tmp2,3))
            tmpname = tmp1||'.'||tmp2
-- If the first 3 chars after the '.' are not a tex filetype, discard
            if not (pos(tmp2,tex_file_types) = 0) then
--              Translate from Unix-style to OS/2-style file names
                tex_error_fname = translate(tmpname,'\','/')
--  Now, name the name fully qualified
--              If only a stem.ext
                if lastpos('\',tex_error_fname) = 0 then
                    tex_error_fname = dr||dir||tex_error_fname
--                    if tex_debug then sayerror 'tex_error_fname (again1)='tex_error_fname; endif
                else
--                  if no drive letter
                    if leftstr(tex_error_fname,1) = '\' then
                        tex_error_fname = dr||tex_error_fname
--                        if tex_debug then sayerror 'tex_error_fname (again2)='tex_error_fname; endif
                    endif
                endif
                name_count = name_count + 1
--                if tex_debug then  sayerror 'Putting 'tex_error_fname' on stack with par_count='par_count' at position 'name_count; endif
                do_array 2, EPM_utility_array_ID, 'tex_fname.'name_count, tex_error_fname
                do_array 2, EPM_utility_array_ID, 'tex_par.'name_count, par_count
                flag = 1
-- Take the filename out of line for next loop
                line = substr(line,pos(tmpname,line)+length(tmpname))
            endif
        endif
    endif

    return flag

-----------------------------------------------------------------------------
--  A little procedure to waste time.
--  Too bad there isn't a built-in that doesn't chomp CPU.

defproc delay(time)
    max = time * 500
    for i = 1 to max
      j = i
    endfor
